/*
 * sta013.h
 *
 *  Created on: May 14, 2016
 *      Author: Hudo
 */

#ifndef L2_DRIVERS_STA013_H_
#define L2_DRIVERS_STA013_H_

#include "LPC17xx.h"

#ifdef __cplusplus
extern "C" {
#endif

#define STA013_CS_PIN		(1<<2)					// Select the PIN# for Decoder's CS Signal
#define STA013_RESET_PIN	(1<<1)					// Select the PIN# for Decoder's Reset Signal
#define STA013_DATREQ_PIN	(1<<0)					// Select the PIN# for Decoder's DATAREQ Signal
#define CONFIG_STA_PINS()	LPC_PINCON->PINSEL4 &= ~(3 << 0); \
							LPC_PINCON->PINSEL4 &= ~(3 << 2); \
							LPC_PINCON->PINSEL4 &= ~(3 << 4); \
							LPC_GPIO2->FIODIR |= STA013_RESET_PIN;	\
							LPC_GPIO2->FIODIR |= STA013_CS_PIN;	\
							LPC_GPIO2->FIOCLR = STA013_CS_PIN; \
							LPC_GPIO2->FIODIR &= ~STA013_DATREQ_PIN; \
							LPC_GPIO2->FIOSET = STA013_RESET_PIN;

/// These # defines should work.
#define STA013_RESET(yes)   if(yes) LPC_GPIO2->FIOCLR = STA013_RESET_PIN; else LPC_GPIO2->FIOSET = STA013_RESET_PIN
#define SELECT_MP3_CS()		LPC_GPIO2->FIOSET = STA013_CS_PIN
#define DESELECT_MP3_CS()	LPC_GPIO2->FIOCLR = STA013_CS_PIN
#define STA013_NEEDS_DATA()	(LPC_GPIO2->FIOPIN & STA013_DATREQ_PIN)

/// Initializes the MP3 Decoder
/// Performs Reset and downloads the software update.
unsigned char initialize_sta013(void);

void sta013StartDecoder(void);	///< Starts the MP3 Decoding
void sta013StopDecoder(void);	///< Stops the MP3 Decoding
void sta013PauseDecoder(void);	///< Pauses the MP3 Decoder
void sta013ResumeDecoder(void);	///< Resumes the paused MP3 Decoder.

/// Gets the bitrate, sampling frequency and mode for the mp3 data being streamed
void sta013GetMP3Info(unsigned short *bitrate, unsigned char *sampFreq, unsigned char *mode);

/// @returns the average bit-rate of the mp3 data being streamed
unsigned short sta013GetAverageBitrate(void);

/// Sets the Volume 0-100 and the balance of left/right audio channels
void sta013SetVolume(unsigned char volume, char balance);

/// Sets the bass and treble of the Decoder
void sta013SetTone(char bassEnh, unsigned short bassFreq, char trebleEnh, unsigned short trebleFreq);


#ifdef __cplusplus
}
#endif
#endif /* L2_DRIVERS_STA013_H_ */
