/*
 *     SocialLedge.com - Copyright (C) 2016
 *
 *     This file is part of free software framework for embedded processors.
 *     You can use it and/or distribute it as long as this copyright header
 *     remains unmodified.  The code is free for personal use and requires
 *     permission to use in a commercial product.
 *
 *      THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 *      OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 *      MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 *      I SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 *      CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *     You can reach the author of this software at :
 *          p r e e t . w i k i @ g m a i l . c o m
 */
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>

#include "NRFWireless.h"
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"
#include "semphr.h"

#include "nrf24L01Plus.h"
#include "sys_config.h"   /* WIRELESS_CHANNEL_NUM */
#include "lpc_sys.h"
#include "eint.h"

#include "WirelessConfig.h"

#define MESH_PAYLOAD                32

static QueueHandle_t g_rx_queue = NULL;     ///< Queue handle for RX queue
static SemaphoreHandle_t g_nrf_activity_sem = NULL; ///< If FreeRTOS is running, we will not poll for nordic activity

/// ISR callback function upon NRF IRQ rising edge interrupt
static void nrf_irq_callback(void)
{
    long yieldRequired = 0;
    xSemaphoreGiveFromISR(g_nrf_activity_sem, &yieldRequired);
//#if NRF_SENDER
//    nordic_clear_packet_sent_flag();
//#else
//    nordic_clear_packet_available_flag();
//#endif

    portEND_SWITCHING_ISR(yieldRequired);
}
#if 0
int nrf_driver_init(void)
{
    if (NULL == g_rx_queue) {
        g_rx_queue = xQueueCreate(WIRELESS_RX_QUEUE_SIZE, MESH_PAYLOAD);
    }
    if (NULL == g_nrf_activity_sem) {
        g_nrf_activity_sem = xSemaphoreCreateBinary();
    }
	// 				32				2499					2000
    nordic_init(MESH_PAYLOAD, WIRELESS_CHANNEL_NUM, WIRELESS_AIR_DATARATE_KBPS);
    nordic_standby1_to_rx();

    /* Hook up the interrupt callback for nordic pin */
	//							22
    eint3_enable_port0(BIO_NORDIC_IRQ_P0PIN, eint_falling_edge, nrf_irq_callback);

    return (NULL != g_rx_queue && NULL != g_nrf_activity_sem);
}
#endif

#if NRF_SENDER
int nrf_driver_init(void)
{
    if (NULL == g_rx_queue) {
        g_rx_queue = xQueueCreate(WIRELESS_RX_QUEUE_SIZE, MESH_PAYLOAD);
    }
    if (NULL == g_nrf_activity_sem) {
//    	g_nrf_activity_sem = xSemaphoreCreateBinary();
    	g_nrf_activity_sem = xSemaphoreCreateCounting( 3, 3 );
    }
	// 				32				2499					2000
    nordic_init(MESH_PAYLOAD, WIRELESS_CHANNEL_NUM, WIRELESS_AIR_DATARATE_KBPS);
    nordic_standby1_to_tx_mode2();

    /* Hook up the interrupt callback for nordic pin */
	//							22
    eint3_enable_port0(BIO_NORDIC_IRQ_P0PIN, eint_falling_edge, nrf_irq_callback);

    return (NULL != g_rx_queue && NULL != g_nrf_activity_sem);
}

#else

int nrf_driver_init(void)
{
    if (NULL == g_rx_queue) {
        g_rx_queue = xQueueCreate(WIRELESS_RX_QUEUE_SIZE, MESH_PAYLOAD);
    }
    if (NULL == g_nrf_activity_sem) {
//        g_nrf_activity_sem = xSemaphoreCreateBinary();
        g_nrf_activity_sem = xSemaphoreCreateCounting( 3, 0 );
    }
	// 				32				2499					2000
    nordic_init(MESH_PAYLOAD, WIRELESS_CHANNEL_NUM, WIRELESS_AIR_DATARATE_KBPS);
    nordic_standby1_to_rx();

    /* Hook up the interrupt callback for nordic pin */
	//							22
    eint3_enable_port0(BIO_NORDIC_IRQ_P0PIN, eint_falling_edge, nrf_irq_callback);

    return (NULL != g_rx_queue && NULL != g_nrf_activity_sem);
}
#endif

int nrf_driver_send(void* p, int len)
{
    int packetWasSent = 1;

    //const mesh_packet_t *pkt = (mesh_packet_t*)p;
    //const char *pkt = (char*) p;

	// Bring from RX mode to TX mode
	//nordic_rx_to_Stanby1();
	nordic_standby1_to_tx_mode1();

	// Send the packet :
	nordic_mode1_send_single_packet(p, len);
	puts("Sending a byte");
	nordic_clear_packet_sent_flag();

	// Switch back to receive mode
	//nordic_standby1_to_rx();

	/* If FreeRTOS is running, we are probably blocked indefinitely on the activity semaphore.
	 * So we will give the semaphore here, to give the mesh network task to unblock and
	 * carry out retry logic.  We use FromISR() API such that mesh_send() will not be
	 * restricted to be called from a FreeRTOS task alone.
	 */
	if (taskSCHEDULER_RUNNING == xTaskGetSchedulerState()) {
	    xSemaphoreGiveFromISR(g_nrf_activity_sem, NULL);
	}

	return packetWasSent;
}

int nrf_driver_receive(void* p, int len)
{
    int packetWasReceived = 0;
    //puts("Receiver...\n");
    nordic_read_rx_fifo(p, len);
    char temp[3];
    temp[2] = '\n';
    temp[1] = '\0';
    temp[0] = *((char*)p);
    puts(temp);
       /*if(nordic_is_packet_available())
       {
           nordic_read_rx_fifo(p, len);
           puts("Byte received?");

           // Only clear the interrupt if no more packet available
           // because nordic has 3 level Rx FIFO.
           if (!nordic_is_packet_available()) {
               nordic_clear_packet_available_flag();
           }
           packetWasReceived = 1;
       }
       */

       return packetWasReceived;

}

int nrf_driver_app_recv(void *p, int len)
{
    // Use to queue data
    const QueueHandle_t qhandle = g_rx_queue;

    int ok = xQueueSend(qhandle, p, 0);


    return ok;
}

/// Retrieves a packet from the queue handle with the given timeout.
char wireless_get_queued_pkt(QueueHandle_t qhandle, char *pkt, const uint32_t timeout_ms)
{
    char ok = 0;

    if (taskSCHEDULER_RUNNING == xTaskGetSchedulerState()) {
        ok = xQueueReceive(qhandle, pkt, OS_MS(timeout_ms));
    }
    else {
        uint64_t timeout_of_char = sys_get_uptime_ms() + timeout_ms;
        while (! (ok = xQueueReceive(qhandle, pkt, 0))) {
            if (sys_get_uptime_ms() > timeout_of_char) {
                break;
            }
        }
    }

    return ok;
}

void NRFwireless_service(void)
{
    /*
     * If FreeRTOS is running, then a task should be calling us, so we can block on
     * the nordic activity semaphore.
     *
     * There are three cases of block time :
     *  1 - If nordic interrupt signal is still pending, then we haven't read
     *      all Nordic FIFO, so we don't block on semaphore at all.
     *  2 - There are pending packets that need either ACK or retry, so we
     *      block just for one tick to carry out mesh logic.
     *  3 - No RX and no TX, so block until either a packet is sent, or until
     *      we receive a packet; both cases will give the semaphore.
     */
    if (taskSCHEDULER_RUNNING == xTaskGetSchedulerState()) {
        if (!nordic_intr_signal()) {
            //const TickType_t blockTime = mesh_get_pnd_pkt_count() ? 1 : portMAX_DELAY;
			const TickType_t blockTime = portMAX_DELAY;
            xSemaphoreTake(g_nrf_activity_sem, blockTime);
        }
		//Semaphore Give from nrf_driver_send()
    }
}

void SenderWireless_service(void * data, unsigned short length)
{
	xSemaphoreTake(g_nrf_activity_sem, portMAX_DELAY);
//
////	while(nordic_is_tx_fifo_full()) ;
//
	nordic_clear_packet_sent_flag();
//
	nordic_queue_tx_fifo(data, length);

}

void ReceiverWireless_service(void * data, unsigned short length)
{
   //add this service to run
   //Queue needed to handle the send and recieve of data
   //if receiver, monitor the queue to save to SD card --> test to recieve file and write to SD card

	xSemaphoreTake(g_nrf_activity_sem, portMAX_DELAY);

//	while(!nordic_is_packet_available()) ;
	nordic_clear_packet_available_flag();

//	while(nordic_is_rx_fifo_empty()) ;

	nordic_read_rx_fifo(data, length);

}
